@@ -1,7 +1,15 @@
-;; Incremental search for SKK (version 1.3 of March 22, 1990)
+;; Incremental search for SKK (version 1.9 of April 21, 1991)
+
+;; version 1.9 released on April 21, 1991
+;; version 1.8 released on April 18, 1991
+;; version 1.7 released on April 17, 1991
+;; version 1.6 released on April 12, 1991
+;; version 1.5 released on April 11, 1991
+;; version 1.4 released on October 1, 1990
 ;; version 1.3 released on March 22, 1990
 ;; version 1.2 released on April 10, 1989
 ;; Masahiko Sato modified the original incremental search.
+;; Tsugutomo Enami (enami@ptgd.sony.co.jp) contributed modifications.
 
 ;; Incremental search
 ;; Copyright (C) 1985, 1986 Free Software Foundation, Inc.
@@ -57,6 +65,14 @@
 ;This is the style where a one-line window is created to show the line
 ;that the search has reached.")
 
+(defvar j-ignore-exp "[\n\t <>|]*"
+  "*If non-nil, it is inserted between consecutive JIS characters in the
+search string.")
+
+(defvar j-ascii-ignore-exp "[\n\t <>|]*"
+  "*If non-nil, it is inserted between an ASCII character and a space
+charcter.  The space character will be deleted.")
+
 (defun j-eval-one-command (char)
   "Interpret CHAR under the current keymaps getting additional keys from
 the keyboard if CHAR does not correspond to a command."
@@ -74,25 +90,85 @@ the keyboard if CHAR does not correspond to a command."
     (sit-for 3)
     (ding)))
 
-(defun skk-isearch-forward ()
+(defun skk-isearch-forward (&optional prefix)
   "Forward incremental search under SKK"
-  (interactive)
-  (if (and (featurep 'skk) skk-mode) (skk-isearch t) (isearch t)))
+  (interactive "P")
+  ;; if SKK has not been invoked yet in this Emacs session,
+  ;; call it now, and initialize some variables.
+  (or (featurep 'skk)
+      (save-excursion
+	(set-buffer (get-buffer-create " *search-buffer* "))
+	(skk-mode 1)))
+  (let ((j-ignore-exp (if prefix nil j-ignore-exp))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
+	(j-mode (if skk-mode
+		    ;; if SKK is active on this buffer, then use its j-mode
+		    j-mode
+		  ;; otherwise, check if the buffer contains kanji code
+		  (if (check-region-kanji-code (point-min) (point-max))
+		      t
+		    0))))
+    (if (eq j-mode 0) (isearch t) (skk-isearch t))))
 
-(defun skk-isearch-backward ()
+(defun skk-isearch-backward (&optional prefix)
   "Backward incremental search under SKK"
-  (interactive)
-  (if (and (featurep 'skk) skk-mode) (skk-isearch nil) (isearch nil)))
+  (interactive "P")
+  ;; if SKK has not been invoked yet in this Emacs session,
+  ;; call it now, and initialize some variables.
+  (or (featurep 'skk)
+      (save-excursion
+	(set-buffer (get-buffer-create " *search-buffer* "))
+	(skk-mode 1)))
+  (let ((j-ignore-exp (if prefix nil j-ignore-exp))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
+	(j-mode (if skk-mode
+		    ;; if SKK is active on this buffer, then use its j-mode
+		    j-mode
+		  ;; otherwise, check if the buffer contains kanji code
+		  (if (check-region-kanji-code (point-min) (point-max))
+		      t
+		    0))))
+    (if (eq j-mode 0) (isearch nil) (skk-isearch nil))))
 
-(defun skk-isearch-forward-regexp ()
+(defun skk-isearch-forward-regexp (&optional prefix)
   "Forward incremental search for regular expression under SKK"
-  (interactive)
-  (if (and (featurep 'skk) skk-mode) (skk-isearch t t) (isearch t t)))
+  (interactive "P")
+  ;; if SKK has not been invoked yet in this Emacs session,
+  ;; call it now, and initialize some variables.
+  (or (featurep 'skk)
+      (save-excursion
+	(set-buffer (get-buffer-create " *search-buffer* "))
+	(skk-mode 1)))
+  (let ((j-ignore-exp (if prefix nil j-ignore-exp))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
+	(j-mode (if skk-mode
+		    ;; if SKK is active on this buffer, then use its j-mode
+		    j-mode
+		  ;; otherwise, check if the buffer contains kanji code
+		  (if (check-region-kanji-code (point-min) (point-max))
+		      t
+		    0))))
+    (if (eq j-mode 0) (isearch t t) (skk-isearch t t))))
 
-(defun skk-isearch-backward-regexp ()
-  "Forward incremental search for regular expression under SKK"
-  (interactive)
-  (if (and (featurep 'skk) skk-mode) (skk-isearch nil t) (isearch nil t)))
+(defun skk-isearch-backward-regexp (&optional prefix)
+  "Backward incremental search for regular expression under SKK"
+  (interactive "P")
+  ;; if SKK has not been invoked yet in this Emacs session,
+  ;; call it now, and initialize some variables.
+  (or (featurep 'skk)
+      (save-excursion
+	(set-buffer (get-buffer-create " *search-buffer* "))
+	(skk-mode 1)))
+  (let ((j-ignore-exp (if prefix nil j-ignore-exp))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
+	(j-mode (if skk-mode
+		    ;; if SKK is active on this buffer, then use its j-mode
+		    j-mode
+		  ;; otherwise, check if the buffer contains kanji code
+		  (if (check-region-kanji-code (point-min) (point-max))
+		      t
+		    0))))
+    (if (eq j-mode 0) (isearch nil t) (skk-isearch nil t))))
 
 ;; This function does all the work of incremental search under SKK.
 
@@ -101,6 +177,10 @@ the keyboard if CHAR does not correspond to a command."
 	(search-buff " *search-buffer*")
 	(cont nil)
 	(j j-mode)
+	(k j-katakana)
+	(z j-zenkaku)
+	(skk-echo nil) ; inhibit echoing of roman prefix
+	;(skk-isearch-message nil)
 	quote
 	(cmds nil)
 	(success t)
@@ -121,10 +201,16 @@ the keyboard if CHAR does not correspond to a command."
     (save-excursion
       (set-buffer (get-buffer-create search-buff))
       (erase-buffer)
-      (skk-mode)
-      (setq j-mode j)
-      (or j-mode (j-mode-off))
-      (setq j-henkan-on nil))
+      (setq j-mode j
+	    j-katakana k
+	    j-zenkaku z)
+      ;; modified by Tsugutomo Enami, 1990.7.13
+      (skk-mode 1)
+      (j-kakutei)
+      (if j
+	  (if k (j-toggle-kana))
+	(if z (j-zenkaku-eiji)
+	  (j-mode-off))))
     (j-isearch-push-state)
     (save-window-excursion
      (catch 'search-done
@@ -247,13 +333,13 @@ the keyboard if CHAR does not correspond to a command."
 			  (setq unread-command-char char)
 			  (throw 'search-done t))
 			 (t
-			  ;; Any other character => add it to the
-			  ;;  search string and search.
+			  ;; Any other character => interpret the character
+			  ;; in the search-buff.
 			  (if (= char search-quote-char)
 			      (setq char (read-quoted-char
 					  (j-isearch-message t))
 				    quote t))
-			  (let (string)
+			  (let (string (skk-isearch-message t))
 			    (save-excursion
 			      (set-buffer search-buff)
 			      (if quote (insert char)
@@ -289,8 +375,8 @@ the keyboard if CHAR does not correspond to a command."
 			 (cond ((memq char '(?* ??))
 				(setq adjusted t)
 				(let ((cs (nth (if forward
-						   5 ; other-end
-						 2) ; saved (point)
+						   4 ; other-end
+						 1) ; saved (point)
 					       (car (cdr cmds)))))
 				  ;; (car cmds) is after last search;
 				  ;; (car (cdr cmds)) is from before it.
@@ -406,11 +492,18 @@ the keyboard if CHAR does not correspond to a command."
       (let ((inhibit-quit nil))
 	(if regexp (setq invalid-regexp nil))
 	(setq success
-	      (funcall
-	       (if regexp
+	      (if j-ignore-exp
+		  (funcall
 		   (if forward 're-search-forward 're-search-backward)
-		 (if forward 'search-forward 'search-backward))
-	       search-string nil t))
+		   (if regexp
+		       search-string
+		     (j-add-ignore-exp (regexp-quote search-string)))
+		   nil t)
+		(funcall
+		 (if regexp
+		     (if forward 're-search-forward 're-search-backward)
+		   (if forward 'search-forward 'search-backward))
+		 search-string nil t)))
 	(if success
 	    (setq other-end
 		  (if forward (match-beginning 0) (match-end 0)))))
@@ -428,6 +521,38 @@ the keyboard if CHAR does not correspond to a command."
 	 (ding))
     (goto-char (nth 1 (car cmds)))))
 
+(defun j-add-ignore-exp (str)
+  "Expand STR by inserting j-ignore-str between JIS characters."
+  (save-excursion
+    (set-buffer (get-buffer-create " *search-work-buff*"))
+    (erase-buffer)
+    (insert str)
+    (goto-char (point-min))
+    (or (eobp)
+	(forward-char 1)
+	(while (not (eobp))
+	  (if (< (preceding-char) 128)
+	      (if (and (= (following-char) ? ) j-ascii-ignore-exp)
+		  ;; if j-ascii-ignore-exp is non-nil (and in such a case
+		  ;; it must be a string for a regular expression) and
+		  ;; if the preceding char is an ascii char and the following
+		  ;; char is a space then replace it by j-ascii-ignore-exp.
+		  (progn
+		    (insert j-ascii-ignore-exp)
+		    (delete-char 1))
+		;; otherwise go forward
+		(forward-char 1))
+	    (if (>= (following-char) 128)
+		;; if the point is between two JIS characters, insert
+		;; j-ignore-exp.
+		(progn
+		  (insert j-ignore-exp)
+		  (forward-char 1))
+	      (forward-char 1)))))
+    ;; it is necessary to enclose the string by a pair of parentheses
+    ;; to cope with a bug in Nemacs' regexp search
+    (concat "\\(" (buffer-substring (point-min) (point-max)) "\\)")))
+
 ;; This is called from incremental-search
 ;; if the first input character is the exit character.
 ;; The interactive-arg-reader uses free variables `forward' and `regexp'
